@@ -183,6 +183,7 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def __init__(self, parent=None, disable_player=False):
         super().__init__(parent)
+        self._shown = False
         self.selected_objects = []
         self.ignore_selection_changes = IgnoreSelectionContext(self.update_selection)
         self.toolbar = None
@@ -286,8 +287,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
     def showEvent(self, event):
         super().showEvent(event)
-        if DesktopStatusIndicator:
+        if not self._shown and DesktopStatusIndicator:
             self.register_status_indicator(DesktopStatusIndicator(self.windowHandle()))
+        self._shown = True
 
     def closeEvent(self, event):
         config = get_config()
@@ -549,8 +551,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
 
         self.track_search_action = QtWidgets.QAction(icontheme.lookup('system-search'), _("Search for similar tracks..."), self)
         self.track_search_action.setStatusTip(_("View similar tracks and optionally choose a different release"))
-        self.track_search_action.triggered.connect(self.show_more_tracks)
+        self.track_search_action.setEnabled(False)
         self.track_search_action.setShortcut(QtGui.QKeySequence(_("Ctrl+T")))
+        self.track_search_action.triggered.connect(self.show_more_tracks)
 
         self.show_file_browser_action = QtWidgets.QAction(_("File &Browser"), self)
         self.show_file_browser_action.setCheckable(True)
@@ -1127,6 +1130,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         return None
 
     def show_more_tracks(self):
+        if not self.selected_objects:
+            return
         obj = self.selected_objects[0]
         if isinstance(obj, Track) and obj.files:
             obj = obj.files[0]
@@ -1147,7 +1152,9 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         dialog.exec_()
 
     def view_info(self, default_tab=0):
-        if isinstance(self.selected_objects[0], Album):
+        if not self.selected_objects:
+            return
+        elif isinstance(self.selected_objects[0], Album):
             album = self.selected_objects[0]
             dialog = AlbumInfoDialog(album, self)
         elif isinstance(self.selected_objects[0], Cluster):
@@ -1176,6 +1183,8 @@ class MainWindow(QtWidgets.QMainWindow, PreserveGeometry):
         self.tagger.refresh(self.selected_objects)
 
     def browser_lookup(self):
+        if not self.selected_objects:
+            return
         self.tagger.browser_lookup(self.selected_objects[0])
 
     @throttle(100)
